@@ -15,9 +15,15 @@
 template <typename T>
 class SIBOptimizer {
     public:
-        SIBOptimizer(int32_t n_clusters, int32_t n_features);
+        SIBOptimizer(int32_t n_clusters, int32_t n_features, bool fast_log);
         virtual ~SIBOptimizer();
 
+        void init_centroids(
+                int32_t n_samples, const int32_t *xy_indices,
+                const int32_t *xy_indptr, const T *xy_data,
+                const T* x_sum, int32_t *labels,
+                int32_t *t_size, T *t_sum, double *t_log_sum, T *t_centroid);
+
         void iterate(bool clustering_mode,                          // clustering / classification mode
                 int32_t n_samples, const int32_t *xy_indices,       // data to cluster / classify
                 const int32_t *xy_indptr, const T *xy_data,
@@ -31,6 +37,7 @@ class SIBOptimizer {
     private:
         int32_t n_clusters;
         int32_t n_features;
+        double (*log2_ptr)(double);
 };
 
 #endif // SIB_OPTIMIZER_H
